<?php

namespace Cron;

use DateTime;
use DateInterval;

/**
 * Year field.  Allows: * , / -
 *
 * @author Michael Dowling <mtdowling@gmail.com>
 */
class YearField extends AbstractField
{
	/**
	 * {@inheritdoc}
	 */
	public function isSatisfiedBy(DateTime $date, $value)
	{
		return $this->isSatisfied($date->format('Y'), $value);
	}

	/**
	 * {@inheritdoc}
	 */
	public function increment(DateTime $date, $invert = false)
	{
		if ($invert)
		{
			$date->sub(new DateInterval('P1Y'));
			$date->setDate($date->format('Y'), 12, 31);
			$date->setTime(23, 59, 0);
		}
		else
		{
			$date->add(new DateInterval('P1Y'));
			$date->setDate($date->format('Y'), 1, 1);
			$date->setTime(0, 0, 0);
		}

		return $this;
	}

	/**
	 * {@inheritdoc}
	 */
	public function validate($value)
	{
		return (bool) preg_match('/[\*,\/\-0-9]+/', $value);
	}
}