<?php
/**
 * HUBzero CMS
 *
 * Copyright 2005-2013 Purdue University. All rights reserved.
 *
 * This file is part of: The HUBzero(R) Platform for Scientific Collaboration
 *
 * The HUBzero(R) Platform for Scientific Collaboration (HUBzero) is free
 * software: you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * HUBzero is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * HUBzero is a registered trademark of Purdue University.
 *
 * @package   hubzero-cms
 * @author    Shawn Rice <zooley@purdue.edu>
 * @copyright Copyright 2005-2013 Purdue University. All rights reserved.
 * @license   http://www.gnu.org/licenses/lgpl-3.0.html LGPLv3
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

require_once(__DIR__ . '/abstract.php');

/**
 * Adapter class for a forum post link for group forum
 */
class ForumModelAdapterGroup extends ForumModelAdapterAbstract
{
	/**
	 * URL segments
	 *
	 * @var string
	 */
	protected $_segments = array(
		'option' => 'com_groups',
	);

	/**
	 * Constructor
	 *
	 * @param   integer $scope_id Scope ID (group, course, etc.)
	 * @return  void
	 */
	public function __construct($scope_id)
	{
		$group = \Hubzero\User\Group::getInstance($scope_id);

		if (!$group)
		{
			$group = new \Hubzero\User\Group();
			$group->set('cn', '_unknown_');
		}

		$this->_segments['cn']     = $group->get('cn');
		$this->_segments['active'] = 'forum';

		$this->_name = $group->get('cn');
	}

	/**
	 * Generate and return various links to the entry
	 * Link will vary depending upon action desired, such as edit, delete, etc.
	 *
	 * @param   string $type   The type of link to return
	 * @param   mixed  $params Optional string or associative array of params to append
	 * @return  string
	 */
	public function build($type='', $params=null)
	{
		$segments = $this->_segments;

		if ($this->get('section'))
		{
			$segments['scope'] = $this->get('section');
		}
		if ($this->get('category'))
		{
			$segments['scope'] .= '/' . $this->get('category');
		}

		$anchor = '';

		// If it doesn't exist or isn't published
		switch (strtolower($type))
		{
			case 'base':
				return $this->_base . '?' . (string) $this->_build($segments);
			break;

			case 'edit':
				if ($this->get('post'))
				{
					$segments['scope'] .= '/' . $this->get('post');
				}
				$segments['scope'] .= '/edit';
			break;

			case 'delete':
				if ($this->get('post'))
				{
					$segments['scope'] .= '/' . $this->get('post');
				}
				$segments['scope'] .= '/delete';
			break;

			case 'new':
			case 'newthread':
				$segments['scope'] .= '/new';
			break;

			case 'download':
				if ($this->get('thread'))
				{
					$segments['scope'] .= '/' . $this->get('thread');
				}
				$segments['scope'] .= '/' . $this->get('post') . '/';
			break;

			case 'reply':
				if ($this->get('thread'))
				{
					$segments['scope'] .= '/' . $this->get('thread');
				}
				$segments['reply'] = $this->get('post');
			break;

			case 'anchor':
				if ($this->get('post'))
				{
					$anchor = '#c' . $this->get('post');
				}
			break;

			case 'abuse':
				return 'index.php?option=com_support&task=reportabuse&category=forum&id=' . $this->get('post') . '&parent=' . $this->get('parent');
			break;

			case 'permalink':
			default:
				if ($this->get('thread'))
				{
					$segments['scope'] .= '/' . $this->get('thread');
				}
			break;
		}

		if (is_string($params))
		{
			$params = str_replace('&amp;', '&', $params);

			if (substr($params, 0, 1) == '#')
			{
				$anchor = $params;
			}
			else
			{
				if (substr($params, 0, 1) == '?')
				{
					$params = substr($params, 1);
				}
				parse_str($params, $parsed);
				$params = $parsed;
			}
		}

		$segments = array_merge($segments, (array) $params);

		return $this->_base . '?' . (string) $this->_build($segments) . (string) $anchor;
	}
}

